--[[
Craftaholic 
Version: Beta 1.01
Game Version: 3.2.0
Copyright 2009 Alex DiMarzo
Thegreatiam, Hyjal

This file is part of Craftoholic.

Craftoholic is free software: you can redistribute it and/or modify
it under the terms of version 3 of the GNU General Public License 
as published by the Free Software Foundation.

Craftoholic is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Craftoholic.  If not, see <http://www.gnu.org/licenses/>.
]]--

-- Set varibles for the script; do they need to be global?
CH_MaxResponses = 30
CH_REALMNAME = GetRealmName()
CH_PLAYERNAME = UnitName("player")

-- Create tables for later use
if CH_ProfessionTable == nil then CH_ProfessionTable = {} end
if CH_ProfessionTable[CH_REALMNAME] == nil then CH_ProfessionTable[CH_REALMNAME] = {} end
if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] = {} end
local CH_Professions = {}

local function CH_Respond(MSG,TAR) -- Simplifies the process of sending responses
	SendChatMessage(MSG,"WHISPER",nil,TAR)
end

local function CH_OMGHide(self,event,xmsg,TAR) -- Filters out Craftoholic-sent messages and prevents double-printing when whispering yourself.
	msg = strlower(xmsg)
		if (select(2,strfind(msg,"ch: ")) == 4) then
			return true;
		elseif (select(2,strfind(msg,"!")) == 1) then
			if TAR == CH_PLAYERNAME then return true; end
		else
			return false;
		end -- ==> if (select(...
end -- ==> function OMGHide

local function CH_GetCharProfs() -- Puts the current character's professions into the table CH_Professions
	local NumSkillLines = GetNumSkillLines()
	for SkillIndex = 1,NumSkillLines do -- 
		CollapseSkillHeader(SkillIndex)
	end
	ExpandSkillHeader(2)
	CH_Professions = {GetSkillLineInfo(3),GetSkillLineInfo(4)}
	for SkillIndex = NumSkillLines,1,-1 do
		ExpandSkillHeader(SkillIndex)
	end
end -- ==> function GetCharProfs

local function CH_ProfScan() -- Gathers information about profession and stores recipes & item tooltips for use in searches
	local Profession,Skill,Max = GetTradeSkillLine()
	local ListLink = GetTradeSkillListLink()
	
	-- Check for others' professions & (for now) secondary skills
	if strmatch("First Aid Cooking Smelting",Profession) then return 1 end
	if ListLink == nil then return 1 end
	if IsTradeSkillLinked() == 1 then return 1 end
	if GetSpellLink(Profession) == nil then return 1 end -- *** TO DO *** set up to delete entries in the DB that are no longer known.
	
	-- Create tables if needed
	if CH_ProfessionTable == nil then CH_ProfessionTable = {} end
	if CH_ProfessionTable[CH_REALMNAME] == nil then CH_ProfessionTable[CH_REALMNAME] = {} end
	if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] = {} end
	if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] = {} end
		
	-- Enter values into table
	CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession][1]=Profession
	CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession][2]=Skill
	CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession][3]=Max
	CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession][4]=select(2,GetSpellLink(Profession))

	CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession]["Recipes"] = {} -- Clear current Recipe DB
	
	for i=GetNumTradeSkills(),1,-1 do if GetTradeSkillItemLink(i)== nil then ExpandTradeSkillSubClass(i) end end -- Expand all headers
	
	-- :: Scan and record tooltip ::
	for i=1,GetNumTradeSkills() do
		if GetTradeSkillItemLink(i) ~= nil then
			local WinString = ""
			MyScanningTooltip:ClearLines()
			MyScanningTooltip:SetHyperlink(GetTradeSkillItemLink(i))
			for i=1,MyScanningTooltip:NumLines() do
			   local mytext = getglobal("MyScanningTooltipTextLeft" .. i)
			   local text = mytext:GetText()
			   if WinString == "" then WinString = strlower(text) else WinString = WinString.." "..strlower(text) end
			end
			CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession]["Recipes"][WinString] = GetTradeSkillRecipeLink(i)
		end
	end
end -- ==> function CH_ProfScan()

local function CH_FormatSearch(xstring) -- Used by CraftyResponse to match a user's query to a profession.
	local string = strlower(xstring)
	
	if string == nil then return nil
	elseif string == "" then return nil
	elseif string == "all" then return "all"
	elseif string == "full" then return "full"
	elseif string == "help" then return "halp"
	elseif strmatch("alchemy",string) then return "Alchemy"
	elseif strmatch("bs blacksmithing",string) then return "Blacksmithing"
	elseif strmatch("enchanting",string) then return "Enchanting"		
	elseif strmatch("engineering",string) then return "Engineering"
	elseif strmatch("scribe inscription",string) then return "Inscription"
	elseif strmatch("jc gem jewelcrafting",string) then return "Jewelcrafting"
	elseif strmatch("lw leatherworking",string) then return "Leatherworking"
	elseif strmatch("tailoring",string) then return "Tailoring"
	
	------------------------------------
	-- *ADD* Cooking, maybe First Aid --
	------------------------------------

	else return nil
	end -- ==> if string == nil
end -- ==> function CH_FormatSearch

local function CH_FullQuery(profession)  -- Matches query to list of professions for each character on the realm.
local CharString = "" local NumResponses = 0 local ResponseTable = {}
	for Character,_ in pairs(CH_ProfessionTable[CH_REALMNAME]) do
		for ProfName,_ in pairs(CH_ProfessionTable[CH_REALMNAME][Character]) do
			if ProfName == profession then
				NumResponses = NumResponses + 1
				local LINK = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][4]
				local CurrentSkill = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][2]
				local MaxSkill = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][3]
				local ResponseLine = "CH: " .. Character .. " - " .. LINK .. " " .. CurrentSkill .. "/" .. MaxSkill
				tinsert(ResponseTable,ResponseLine)
				if CharString == "" then CharString = Character else if not strmatch(CharString,Character) then CharString = CharString..", "..Character end end
			elseif profession == "all" then
				if CH_ProfessionTable[CH_REALMNAME][Character][ProfName][4] ~= nil then
					NumResponses = NumResponses + 1
					local LINK = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][4]
					local CurrentSkill = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][2]
					local MaxSkill = CH_ProfessionTable[CH_REALMNAME][Character][ProfName][3]
					local ResponseLine = "CH: " .. Character .. " - " .. LINK .. " " .. CurrentSkill .. "/" .. MaxSkill
					tinsert(ResponseTable,ResponseLine)
					if CharString == "" then CharString = Character else if not strmatch(CharString,Character) then CharString = CharString..", "..Character end end
				end -- ==> if CH_ProfessionTable[stuff] ~= nil
			end -- ==> if ProfName == profession
		end -- ==> for ProfName,_
	end -- ==> for Character,_
return ResponseTable, NumResponses, CharString
end -- ==> function GetLinks

local function CH_GetMatches(profession,query)  -- Matches query to tooltip text for a given profession.
local CharString = "" local WhisperTable = {} local NumResponses = 0
local STRINGTABLE = {}
print(profession,query)
string.gsub(query, "([%w%p]+)" , function(x) table.insert(STRINGTABLE,x) return x end)
	for Character,_ in pairs(CH_ProfessionTable[CH_REALMNAME]) do
		for ProfName,_ in pairs(CH_ProfessionTable[CH_REALMNAME][Character]) do
			if ProfName == profession then
				RecipeTable = CH_ProfessionTable[CH_REALMNAME][Character][ProfName]["Recipes"]
				if RecipeTable then
				for CachedText,RecipeLink in pairs(RecipeTable) do 
					SearchText = strlower(CachedText)
					local NumMatches = 0
					for _,value in pairs(STRINGTABLE) do 
						if strmatch(SearchText, value) == value then NumMatches = NumMatches + 1 end
					end
					if NumMatches == #(STRINGTABLE) then
						NumResponses = NumResponses + 1
						local LINK = RecipeLink
						local CHAR = Character
						tinsert(WhisperTable,"CH: "..LINK.." - "..Character)
						if CharString == "" then CharString = Character else if not strmatch(CharString,Character) then CharString = CharString..", "..Character end end
					end
				end end
			end
		end
	end
return WhisperTable, NumResponses, CharString
end

local function CH_GetResponse(msg,customer)
	local _,_,flag,rtype,qtext = strfind(msg, "(%p)(%w+)[%s%p]*(.*)")

	if flag == "!" then
		if rtype then
			if CH_FormatSearch(rtype) == nil then
				CH_Respond('CH: For a full profession link, whisper with !full followed by all or part of a profession name (EG: !full jewelcrafting, !full eng) or for all available profession links use !all or !full all.',customer)
				CH_Respond('CH: For an individual craft link, whisper with !profession <stats, item name, etc> (EG: !enchant shield defense, !jc stam str).',customer)
			elseif CH_FormatSearch(rtype) == "full" then
				local query = CH_FormatSearch(qtext)
				local ResponseTable,NumResponses,CharString = CH_FullQuery(query)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			elseif CH_FormatSearch(rtype) == "all" then
				local ResponseTable,NumResponses,CharString = CH_FullQuery(rtype)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			elseif CH_FormatSearch(rtype) == "halp" then
				CH_Respond('CH: For a full profession link, whisper with !full followed by all or part of a profession name (EG: !full jewelcrafting, !full eng) or for all available profession links use !all or !full all.',customer)
				CH_Respond('CH: For an individual craft link, whisper with !profession <stats, item name, etc> (EG: !enchant shield defense, !jc stam str).',customer)
			else
				local profession = CH_FormatSearch(rtype)
				local ResponseTable,NumResponses,CharString = CH_GetMatches(profession,qtext)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			
			end -- ==> if CH_FormatSearch(rtype) == nil
		end -- ==> if rtype then
	end -- ==> if flag == "!"
end -- ==> function CH_GetResponses


local function CH_CraftyResponse(self,event,...)  -- Event handler
	if event == "CHAT_MSG_WHISPER" then
		xmsg,customer = ...
		msg = strlower(xmsg)
			local start,_,flag,rtype,qtext = strfind(msg, "(%p)([%w%p]+)[%s%p]*(.*)")  -- rtype, qtext, customer
		if start == 1 then if flag == "!" then if rtype then
			
			if CH_FormatSearch(rtype) == nil then
				CH_Respond('CH: For a full profession link, whisper with !full followed by all or part of a profession name '..
				'(EG: !full jewelcrafting, !full eng) or for all available profession links use !all or !full all.',customer)
				CH_Respond('CH: For an individual craft link, whisper with !profession <stats, item name, etc> '..
				'(EG: !enchant shield defense, !jc stam str).',customer)
			
			elseif CH_FormatSearch(rtype) == "full" then -- == *** TO DO *** find a way to combine this condition with the next one
				local query = CH_FormatSearch(qtext)
				local ResponseTable,NumResponses,CharString = CH_FullQuery(query)
				if NumResponses == 0 then 
					print("B"..customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." links for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
			
			elseif CH_FormatSearch(rtype) == "all" then
				local ResponseTable,NumResponses,CharString = CH_FullQuery(rtype)
				if NumResponses == 0 then 
					print("B"..customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." links for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
			
			else
				local profession = CH_FormatSearch(rtype)
				local ResponseTable,NumResponses,CharString = CH_GetMatches(profession,qtext)
				if NumResponses == 0 then 
					print("B"..customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." links for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
				
			end -- ==> if CH_FormatSearch(rtype) == nil
		end -- ==> if rtype then
		end -- ==> if flag == "!"
		end -- ==> if start == 1

	elseif event == "TRADE_SKILL_SHOW" then
		CH_GetCharProfs()
		CH_ProfScan()
		
		
	end -- ==> if event == "CHAT_MSG_WHISPER"
 end -- ==> function CraftyResponse
 

CH_EventFrame = CreateFrame("FRAME","CH_Event_Frame")
CH_EventFrame:RegisterEvent("CHAT_MSG_WHISPER")
CH_EventFrame:RegisterEvent("TRADE_SKILL_SHOW")
CH_EventFrame:SetScript("OnEvent",CH_CraftyResponse)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM",CH_OMGHide)